@@ -24,10 +24,12 @@
 #include  "fstream_unicode.h"
 #include  <sstream>
 #include  <iomanip>
+#include  <cstdio>
 
 #include  <boost/version.hpp>
 
 #include  <QDesktopServices>
+#include  <QTableView>
 
 #ifndef WIN32
     #include  <QDir>
@@ -65,7 +67,6 @@ void assertBreakpoint()
 }
 
 
-
 void appendFilePart(istream& in, ostream& out, streampos pos, streamoff nSize)
 {
     const int BFR_SIZE (1024*128);
@@ -76,13 +77,18 @@ void appendFilePart(istream& in, ostream& out, streampos pos, streamoff nSize)
     for (; nSize > 0;)
     {
         streamoff nCrtRead (nSize > BFR_SIZE ? BFR_SIZE : nSize);
-        CB_CHECK1 (nCrtRead == read(in, pBfr, nCrtRead), EndOfFile());
+        CB_CHECK (nCrtRead == read(in, pBfr, nCrtRead), EndOfFile);
         out.write(pBfr, nCrtRead);
 
         nSize -= nCrtRead;
     }
 
-    CB_CHECK1 (out, WriteError());
+    if (!out)
+    {
+        TRACER("appendFilePart() failed");
+    }
+
+    CB_CHECK (out, WriteError);
 }
 
 
@@ -498,7 +504,7 @@ namespace
                 }
                 break;
 
-            default: throw 1; // it should have thrown before getting here
+            default: CB_THROW(CbRuntimeError); // it should have thrown before getting here
             }
         }
 
@@ -526,7 +532,7 @@ namespace
             m_nSize = (MPEG1 == m_eVersion ? 144*m_nBitrate/m_nFrequency + m_nPadding : 72*m_nBitrate/m_nFrequency + m_nPadding);
             break;
 
-        default: throw 1; // it should have thrown before getting here
+        default: CB_THROW(CbRuntimeError); // it should have thrown before getting here
         }
 
         return "";
@@ -680,7 +686,7 @@ void writeZeros(ostream& out, int nCnt)
         out.write(&c, 1);
     }
 
-    CB_CHECK1 (out, WriteError());
+    CB_CHECK (out, WriteError);
 }
 
 
@@ -798,8 +804,8 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
                 cout << endl;
 
 #endif
-                char szBfr[5000] = "mP3DiAgS";
-                szBfr[0] = 0;
+                //char szBfr[5000] = "mP3DiAgS";
+                //szBfr[0] = 0;
 
 
                 strCmdLineName += "/cmdline";
@@ -826,7 +832,7 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
                     // for i in `ls /proc | grep '^[0-9]'` ; do a=`cat /proc/$i/cmdline 2>/dev/null` ; echo $a | grep kde4/libexec ; done
                     if (string::npos != strBfr.find("kde4/libexec/start")) // ttt2 probably works only on Suse and only in some cases
                     {
-                        bIsKde4 = true;
+                        bIsKde4 = true; //ttt9 update
                     }
                 }//*/
 
@@ -909,7 +915,7 @@ Ideally a modal dialog should minimize its parent. If that's not possible, it sh
 //ttt0 look at Qt::CustomizeWindowHint
 #ifndef WIN32
     //Qt::WindowFlags getMainWndFlags() { return isRunningOnGnome() ? Qt::Window : Qt::WindowTitleHint; } // !!! these are incorrect, but seem the best option; the values used for Windows are supposed to be OK; they work as expected with KDE but not with Gnome (asking for maximize button not only fails to show it, but causes the "Close" button to disappear as well); Since in KDE min/max buttons are shown when needed anyway, it's sort of OK // ttt2 see if there is workaround/fix
-    Qt::WindowFlags getMainWndFlags() { const DesktopDetector& dd = getDesktopDetector(); return dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint : Qt::Window; }
+    Qt::WindowFlags getMainWndFlags() { /*const DesktopDetector& dd = getDesktopDetector();*/ return /*dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint | Qt::WindowCloseButtonHint :*/ Qt::Window; }
 #if QT_VERSION >= 0x040500
     //Qt::WindowFlags getDialogWndFlags() { const DesktopDetector& dd = getDesktopDetector(); return dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint : (dd.onDesktop(DesktopDetector::Gnome3) ? Qt::Window : Qt::WindowTitleHint); }
     Qt::WindowFlags getDialogWndFlags() { const DesktopDetector& dd = getDesktopDetector(); return dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint : (/*dd.onDesktop(DesktopDetector::Gnome3) ? Qt::Window :*/ Qt::WindowTitleHint); }
@@ -920,9 +926,11 @@ Ideally a modal dialog should minimize its parent. If that's not possible, it sh
 #endif
     Qt::WindowFlags getNoResizeWndFlags() { return Qt::WindowTitleHint; }
 #else
-    Qt::WindowFlags getMainWndFlags() { return Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowMinimizeButtonHint; } // minimize, maximize, no "what's this"
-    Qt::WindowFlags getDialogWndFlags() { return Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint; } // minimize, no "what's this"
-    Qt::WindowFlags getNoResizeWndFlags() { return Qt::WindowTitleHint; } // no "what's this"
+    Qt::WindowFlags getMainWndFlags() { 
+		return Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowMinimizeButtonHint | Qt::WindowCloseButtonHint;
+	} // minimize, maximize, no "what's this"
+    Qt::WindowFlags getDialogWndFlags() { return Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint; } // minimize, no "what's this"
+    Qt::WindowFlags getNoResizeWndFlags() { return Qt::WindowTitleHint; } // no "what's this" // ttt2 review if we want Qt::WindowCloseButtonHint
 #endif
 
 
@@ -1148,7 +1156,7 @@ vector<QString> getLocalHelpDirs()
         //s_v.push_back("/home/ciobi/cpp/Mp3Utils/mp3diags/trunk/mp3diags/doc/html/");
         s_v.push_back(QString("/usr/share/") + getHelpPackageName() + "-doc/html/"); //ttt0 lower/uppercase variations
         s_v.push_back(QString("/usr/share/doc/") + getHelpPackageName() + "/html/");
-        s_v.push_back(QString("/usr/share/doc/") + getHelpPackageName() + "-1.2.01/html/");
+        s_v.push_back(QString("/usr/share/doc/") + getHelpPackageName() + "-1.4.01/html/");
 #else
         wchar_t wszModule [200];
         int nRes (GetModuleFileName(0, wszModule, 200));
@@ -1273,6 +1281,10 @@ const string& getDesktopIntegrationDir()
         {
             createDir(s_s);
         }
+        catch (const exception& ex)
+        {
+            cerr << "failed to create dir " << s_s << "; reason: " << ex.what() << endl;
+        }
         catch (...)
         { // nothing; this will cause shell integration to be disabled
             cerr << "failed to create dir " << s_s << endl;
@@ -1406,6 +1418,9 @@ public:
             {
                 deleteFile(m_strFileName);
             }
+            catch (const exception&)
+            { //ttt2 do something
+            }
             catch (...)
             { //ttt2 do something
             }
@@ -1697,11 +1712,17 @@ bool deleteKey(const char* szPath, const char* szSubkey)
 Tracer::Tracer(const std::string& s) : m_s(s)
 {
     traceToFile("> " + s, 1);
+#ifdef OUTPUT_TRACE_TO_CONSOLE
+    qDebug("> %s", m_s.c_str());
+#endif
 }
 
 Tracer::~Tracer()
 {
     traceToFile(" < " + m_s, -1);
+#ifdef OUTPUT_TRACE_TO_CONSOLE
+    qDebug("< %s", m_s.c_str());
+#endif
 }
 
 
@@ -1721,6 +1742,126 @@ LastStepTracer::~LastStepTracer()
 //=============================================================================================
 //=============================================================================================
 
+// Patch by Elbert Pol, to make the linker work on OS/2. Not sure why adding "LIBS += -lrt" to src.pro didn't seem to work.
+# if defined OS2
+inline int
+clock_gettime(int clock_id, struct timespec *ts)
+{
+     struct timeval tv;
+
+#if 0
+     if (clock_id != CLOCK_REALTIME) {
+         errno = EINVAL;
+         return (-1);
+     }
+#endif
+     if (gettimeofday(&tv, NULL) < 0)
+         return (-1);
+     ts->tv_sec = tv.tv_sec;
+     ts->tv_nsec = tv.tv_usec * 1000;
+     return (0);
+}
+#endif
+
+int64_t CB_LIB_CALL Timer::getCrtTime() const // returns time in nanoseconds
+{
+#ifdef _WIN32
+    LARGE_INTEGER li;
+    QueryPerformanceCounter(&li);
+    return li.QuadPart*m_nDurMul;
+#else
+    timespec ts;
+    //clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &ts); // CLOCK_MONOTONIC
+    clock_gettime(CLOCK_REALTIME, &ts);
+    return ts.tv_sec*1000000000LL + ts.tv_nsec;
+#endif
+}
+
+
+CB_LIB_CALL Timer::Timer(bool bStart /*= true*/) : m_nStart(-1), m_nFinish(-1), m_pStoredDuration(0)
+{
+#ifdef _WIN32
+    LARGE_INTEGER li;
+    if (QueryPerformanceFrequency(&li))
+    {
+        m_nDurMul = (int64_t)(1e9/li.QuadPart);
+    }
+    else
+    {
+        m_nDurMul = 0;
+    }
+#endif
+    if (bStart) start();
+}
+
+CB_LIB_CALL Timer::Timer(int64_t& storedDuration, bool bStart /*= true*/) : m_nStart(-1), m_nFinish(-1), m_pStoredDuration(&storedDuration)
+{
+#ifdef _WIN32
+    LARGE_INTEGER li;
+    if (QueryPerformanceFrequency(&li))
+    {
+        m_nDurMul = (int64_t)(1e9/li.QuadPart);
+    }
+    else
+    {
+        m_nDurMul = 0;
+    }
+#endif
+    if (bStart) start();
+}
+
+
+Timer::~Timer() {
+    if (m_pStoredDuration != 0) {
+        if (m_nFinish == -1) {
+            *m_pStoredDuration = stop();
+        } else {
+            *m_pStoredDuration = getDuration();
+        }
+    }
+}
+
+/*static*/ std::string CB_LIB_CALL Timer::addThSep(int64_t nTime) // to be used when converting to milli- / micro- seconds
+{
+    char a [25];
+    //sprintf(a, "%f", double(nTime)); //
+    //sprintf(a, "%lld", nTime);
+    sprintf(a, "%ld", nTime);//ttt00 warning in Ubuntu 9.04 due to type mismatch
+    int n ((int)strlen(a));
+    std::string strRes;
+    for (int i = 0; i < n; ++i)
+    {
+        if (!strRes.empty() && 0 == (n - i)%3) { strRes += ','; } //ttt2 assumes that "a" doesn't have separators already
+        strRes += a[i];
+    }
+    return strRes;
+}
+
+
+/*static*/ std::string Timer::getLongFmt(int64_t dur) {
+    int msec (dur / 1000000);
+    int sec (msec / 1000); msec %= 1000;
+    int min (sec / 60); sec %= 60;
+    int hrs (min / 60); min %= 60;
+    char a[40];
+    sprintf(a, "%d:%02d:%02d.%03d", hrs, min, sec, msec);
+    return a;
+}
+
+//=============================================================================================
+//=============================================================================================
+//=============================================================================================
+
+void decreaseRowHeaderFont(QTableView& qtableView)
+{
+#ifdef WIN32
+	QFont font(qtableView.verticalHeader()->font());
+	auto sz(font.pointSizeF());
+	font.setPointSizeF(sz * 0.85);
+	qtableView.verticalHeader()->setFont(font);
+#endif
+}
+
 
 //ttt2 F1 help was very slow on XP once, not sure why; later it was OK
 
