@@ -115,10 +115,18 @@ struct Mp3TransformThread : public PausableThread
 
             notif.setSuccess(m_mp3TransformerGui.transform());
         }
+        catch (const exception& ex)
+        {
+            TRACER1("Mp3TransformThread::run()", 1);
+            TRACER1(ex.what(), 2);
+            CB_ASSERT1 (false, ex.what());
+        }
         catch (...)
         {
-            LAST_STEP("Mp3TransformThread::run()");
+            TRACER("Mp3TransformThread::run() - unknown exception");
             CB_ASSERT (false);
+            /*ttt0 2014.09.23 - if transformations leak exceptions it usually gets here; they should be derived from exception (or CbException), in which case they would be caught and it wouldn't reach this point.
+             */
         }
     }
 
@@ -181,7 +189,7 @@ public:
         catch (const CannotRenameFile&)
         {
             revert();
-            throw CannotDeleteFile();
+            CB_THROW(CannotDeleteFile);
         }
     }
 
@@ -193,6 +201,9 @@ public:
         {
             renameFile(m_strChangedName, m_strOrigName);
         }
+        catch (const exception&)
+        { //ttt2
+        }
         catch (...)
         { //ttt2 perhaps do something
         }
@@ -240,6 +251,9 @@ public:
         {
             deleteFile(m_strName);
         }
+        catch (const exception&)
+        { //ttt2
+        }
         catch (...)
         { //ttt2 perhaps do something
         }
@@ -316,6 +330,8 @@ bool Mp3Transformer::transform()
                 long long nSize, nOrigTime;
                 getFileInfo(strOrigName, nOrigTime, nSize);
 
+                bool bErrorInTransform (false);
+
                 for (int j = 0, m = cSize(m_vpTransf); j < m; ++j)
                 {
                     //TRACER1A("transf ", 4);
@@ -325,27 +341,49 @@ bool Mp3Transformer::transform()
                     //emit stepChanged(l, i + 1);
                     emitStepChanged(l, i + 1);
                     Transformation::Result eTransf;
+                    int nRetryCount (0);
+                e1:
                     try
                     {
                     //TRACER1A("transf ", 5);
                         eTransf = t.apply(*pNewHndl, m_transfConfig, strOrigName, strTempName);
                         //TRACER1A("transf ", 6);
+                        if (nRetryCount > 0)
+                        {
+                            TRACER("YYYYYYYYYYYYY Write retry succeeded for the second time to " + strTempName + ". Exiting to capture this ...");
+                            CB_THROW(WriteError); //ttt0 remove
+                        }
                     }
                     catch (const WriteError&)
                     {
         //qDebug("disk err");
         //TRACER1A("transf ", 7);
-                        m_strErrorFile = strTempName;
-                        m_bWriteError = true;
-                        if (pNewHndl.get() == pOrigHndl)
-                        {
-                        //TRACER1A("transf ", 8);
-                            pNewHndl.release();
-                        }
+                        TRACER1("YYYYYYYYYYYYY Error copying from file " + strOrigName, 1);
+                        TRACER1("YYYYYYYYYYYYY Error writing to file " + strTempName, 3);
+
                         //TRACER1A("transf ", 9);
                         TempFileEraser er (strTempName);
                         //TRACER1A("transf ", 10);
-                        return false; //ttt2 review what happens to pNewHndl
+                        if (nRetryCount < 1)
+                        {
+                            TRACER("Retrying writing to file " + strTempName);
+                            ++nRetryCount;
+                            PausableThread::msleep(30);
+                            strTempName.clear();
+                            goto e1;
+                        }
+                        else
+                        {
+                            m_strErrorFile = strTempName;
+                            m_bWriteError = true;
+                            if (pNewHndl.get() == pOrigHndl)
+                            {
+                            //TRACER1A("transf ", 8);
+                                pNewHndl.release();
+                            }
+                            TRACER1("Too many errors trying to write to file " + strTempName + ". Aborting ...", 2);
+                            return false; //ttt2 review what happens to pNewHndl
+                        }
                     }
                     catch (const EndOfFile&) //ttt2 catch other exceptions, perhaps in the outer loop
                     {
@@ -361,12 +399,28 @@ bool Mp3Transformer::transform()
                         //TRACER1A("transf ", 13);
                         return false;
                     }
+                    catch (const exception& ex)
+                    {
+                        m_strErrorFile = strOrigName;
+                        m_bWriteError = false;
+                        m_strOtherError = ex.what();
+                        if (pNewHndl.get() == pOrigHndl)
+                        {
+                        //TRACER1A("transf ", 121);
+                            pNewHndl.release();
+                        }
+                        TempFileEraser er (strTempName);
+                        //TRACER1A("transf ", 131);
+                        //ttt1 A further improvement would be to have a means to "continue/ignore"
+                        return false;
+                    }
+
 //TRACER1A("transf ", 14);
                     //cout << "trying to apply " << t.getActionName() << " to " << pNewHndl.get()->getName() << endl;
                     if (eTransf != Transformation::NOT_CHANGED)
                     {
                     //TRACER1A("transf ", 15);
-                        CB_ASSERT (!m_pCommonData->m_strTransfLog.empty()); //ttt0 triggered according to http://sourceforge.net/apps/mantisbt/mp3diags/view.php?id=45 ; however, the code is quite simple and it doesn't seem to be a valid reason for m_strTransfLog to be empty (aside from corrupted memory) ; according to https://sourceforge.net/apps/mantisbt/mp3diags/view.php?id=49 removing the .dat file solved the issue
+                        CB_ASSERT (!m_pCommonData->m_strTransfLog.empty()); //ttt0 triggered according to https://sourceforge.net/p/mp3diags/tickets/45/ ; however, the code is quite simple and it doesn't seem to be a valid reason for m_strTransfLog to be empty (aside from corrupted memory) ; according to https://sourceforge.net/p/mp3diags/tickets/49/ removing the .dat file solved the issue
                         if (0 != m_pLog)
                         {
                             (*m_pLog) << "applied " << t.getActionName() << " to " << pNewHndl.get()->getName() << endl;
@@ -398,7 +452,17 @@ bool Mp3Transformer::transform()
                         }
 //TRACER1A("transf ", 21);
                         strPrevTempName = strTempName;
-                        pNewHndl.reset(new Mp3Handler(strTempName, m_pCommonData->m_bUseAllNotes, m_pCommonData->getQualThresholds())); //ttt2 try..catch
+                        try
+                        {
+                            pNewHndl.reset(Mp3Handler::create(strTempName, m_pCommonData->m_bUseAllNotes, m_pCommonData->getQualThresholds()));
+                        }
+                        catch (const Mp3Handler::FileNotFound&)
+                        {
+                            // this shouldn't really happen but apparently it did (email on 2016.03.05); it's probably caused by the same thing that makes external files unavailable sometimes
+                            bErrorInTransform = true; //ttt2 maybe need to catch other exceptions
+                            break;
+                        }
+
                         checkPause();
                         //TRACER1A("transf ", 22);
                         if (isAborted())
@@ -426,113 +490,115 @@ bool Mp3Transformer::transform()
                 string strNewOrigName;  // new name for the orig file; if this is empty, the original file wasn't changed; if it's "*", it was erased; if it's something else, it was renamed;
                 string strProcName;     // name for the proc file; if this is empty, a proc file doesn't exist; if it's something else, it's the file name;
 
-                bool bErrorInTransform (false);
-
                 FileEraser fileEraser;
                 //TRACER1A("transf ", 29);
 
-                try
+                if (!bErrorInTransform)
                 {
-                    //bool bChanged (true);
-                    if (pNewHndl.get() == pOrigHndl)
-                    { // nothing changed
-                    //TRACER1A("transf ", 30);
-                        pNewHndl.release();
-                        //TRACER1A("transf ", 31);
-                        switch (m_transfConfig.getUnprocOrigAction())
-                        {
-                        case TransfConfig::ORIG_DONT_CHANGE: break;
-                        case TransfConfig::ORIG_ERASE: { strNewOrigName = "*"; fileEraser.erase(strOrigName); } break; //ttt2 try ...
-                        case TransfConfig::ORIG_MOVE: { m_transfConfig.getUnprocOrigName(strOrigName, strNewOrigName); renameFile(strOrigName, strNewOrigName); } break;
-                        default: CB_ASSERT (false);
-                        }
-                        //TRACER1A("transf ", 32);
-                    }
-                    else
-                    { // at least a processed file exists
-                    //TRACER1A("transf ", 33);
-                        CB_ASSERT (!strTempName.empty());
-//TRACER1A("transf ", 34);
-                        TempFileEraser tmpEraser (strTempName);
-
-                        // first we have to handle the original file;
-//TRACER1A("transf ", 35);
-                        switch (m_transfConfig.getProcOrigAction())
-                        {
-                        case TransfConfig::ORIG_DONT_CHANGE: break;
-                        case TransfConfig::ORIG_ERASE: { strNewOrigName = "*"; fileEraser.erase(strOrigName); } break; //ttt2 try ...
-                        case TransfConfig::ORIG_MOVE: { m_transfConfig.getProcOrigName(strOrigName, strNewOrigName); renameFile(strOrigName, strNewOrigName); } break;
-                        case TransfConfig::ORIG_MOVE_OR_ERASE:
+                    try
+                    {
+                        //bool bChanged (true);
+                        if (pNewHndl.get() == pOrigHndl)
+                        { // nothing changed
+                        //TRACER1A("transf ", 30);
+                            pNewHndl.release();
+                            //TRACER1A("transf ", 31);
+                            switch (m_transfConfig.getUnprocOrigAction())
                             {
-                                m_transfConfig.getProcOrigName(strOrigName, strNewOrigName);
-                                if (fileExists(strNewOrigName))
-                                {
-                                    strNewOrigName = "*";
-                                    fileEraser.erase(strOrigName);
-                                }
-                                else
-                                {
-                                    renameFile(strOrigName, strNewOrigName);
-                                }
+                            case TransfConfig::ORIG_DONT_CHANGE: break;
+                            case TransfConfig::ORIG_ERASE: { strNewOrigName = "*"; fileEraser.erase(strOrigName); } break; //ttt2 try ...
+                            case TransfConfig::ORIG_MOVE: { m_transfConfig.getUnprocOrigName(strOrigName, strNewOrigName); renameFile(strOrigName, strNewOrigName); } break;
+                            default: CB_ASSERT (false);
                             }
-                            break;
-                        default: CB_ASSERT (false);
+                            //TRACER1A("transf ", 32);
                         }
-//TRACER1A("transf ", 36);
-                        // the last processed file exists (usualy in the same folder as the source), its name is in strTempName, and we have to see what to do with it (erase, rename, or copy);
-                        switch (m_transfConfig.getProcessedAction())
-                        {
-                        case TransfConfig::TRANSF_DONT_CREATE: deleteFile(strTempName); break;
-                        case TransfConfig::TRANSF_CREATE:
-                            {//TRACER1A("transf ", 37);
-                            //Tracer t1 (strOrigName);
-                                m_transfConfig.getProcessedName(strOrigName, strProcName);
-                                //Tracer t2 (strProcName);
-                                //TRACER1A("transf ", 371);
-                                switch (m_transfConfig.getTempAction())
+                        else
+                        { // at least a processed file exists
+                        //TRACER1A("transf ", 33);
+                            CB_ASSERT (!strTempName.empty());
+    //TRACER1A("transf ", 34);
+                            TempFileEraser tmpEraser (strTempName);
+
+                            // first we have to handle the original file;
+    //TRACER1A("transf ", 35);
+                            switch (m_transfConfig.getProcOrigAction())
+                            {
+                            case TransfConfig::ORIG_DONT_CHANGE: break;
+                            case TransfConfig::ORIG_ERASE: { strNewOrigName = "*"; fileEraser.erase(strOrigName); } break; //ttt2 try ...
+                            case TransfConfig::ORIG_MOVE: { m_transfConfig.getProcOrigName(strOrigName, strNewOrigName); renameFile(strOrigName, strNewOrigName); } break;
+                            case TransfConfig::ORIG_MOVE_OR_ERASE:
                                 {
-                                case TransfConfig::TRANSF_DONT_CREATE: { /*TRACER1A("transf ", 372);*/ renameFile(strTempName, strProcName); /*TRACER1A("transf ", 373);*/ break; }
-                                case TransfConfig::TRANSF_CREATE: { /*TRACER1A("transf ", 374);*/ copyFile(strTempName, strProcName); /*TRACER1A("transf ", 375);*/ break; }
-                                default: { /*TRACER1A("transf ", 376);*/ CB_ASSERT (false); }
+                                    m_transfConfig.getProcOrigName(strOrigName, strNewOrigName);
+                                    if (fileExists(strNewOrigName))
+                                    {
+                                        strNewOrigName = "*";
+                                        fileEraser.erase(strOrigName);
+                                    }
+                                    else
+                                    {
+                                        renameFile(strOrigName, strNewOrigName);
+                                    }
                                 }
-                                //TRACER1A("transf ", 377);
+                                break;
+                            default: CB_ASSERT (false);
                             }
-                            break;
+    //TRACER1A("transf ", 36);
+                            // the last processed file exists (usualy in the same folder as the source), its name is in strTempName, and we have to see what to do with it (erase, rename, or copy);
+                            switch (m_transfConfig.getProcessedAction())
+                            {
+                            case TransfConfig::TRANSF_DONT_CREATE: deleteFile(strTempName); break;
+                            case TransfConfig::TRANSF_CREATE:
+                                {//TRACER1A("transf ", 37);
+                                //Tracer t1 (strOrigName);
+                                    m_transfConfig.getProcessedName(strOrigName, strProcName);
+                                    //Tracer t2 (strProcName);
+                                    //TRACER1A("transf ", 371);
+                                    switch (m_transfConfig.getTempAction())
+                                    {
+                                    case TransfConfig::TRANSF_DONT_CREATE: { /*TRACER1A("transf ", 372);*/ renameFile(strTempName, strProcName); /*TRACER1A("transf ", 373);*/ break; }
+                                    case TransfConfig::TRANSF_CREATE: { /*TRACER1A("transf ", 374);*/ copyFile(strTempName, strProcName); /*TRACER1A("transf ", 375);*/ break; }
+                                    default: { /*TRACER1A("transf ", 376);*/ CB_ASSERT (false); }
+                                    }
+                                    //TRACER1A("transf ", 377);
+                                }
+                                break;
 
-                        default: CB_ASSERT (false);
+                            default: CB_ASSERT (false);
+                            }
+                            //TRACER1A("transf ", 38);
+
+                            tmpEraser.release();
+                            //TRACER1A("transf ", 39);
                         }
-                        //TRACER1A("transf ", 38);
-
-                        tmpEraser.release();
-                        //TRACER1A("transf ", 39);
+    //TRACER1A("transf ", 40);
+                        fileEraser.finalize();
+                        //TRACER1A("transf ", 41);
                     }
-//TRACER1A("transf ", 40);
-                    fileEraser.finalize();
-                    //TRACER1A("transf ", 41);
+                    catch (const CannotDeleteFile&)
+                    {
+                    //TRACER1A("transf ", 42);
+                        bErrorInTransform = true;
+                    }
+                    catch (const CannotRenameFile&) //ttt2 perhaps also NameNotFound, AlreadyExists, ...
+                    {
+                    //TRACER1A("transf ", 43);
+                        bErrorInTransform = true;
+                    }
+                    catch (const CannotCreateDir& ex)
+                    {
+                    //TRACER1A("transf ", 44);
+                        bErrorInTransform = true;
+                        m_strErrorDir = ex.m_strDir;
+                    }
+                    catch (const CannotCopyFile& ex)
+                    {
+                    //TRACER1A("transf ", 45);
+                        CB_ASSERT1(false, ex.what());
+                        //bErrorInTransform = true;
+                    }
+    //TRACER1A("transf ", 46);
                 }
-                catch (const CannotDeleteFile&)
-                {
-                //TRACER1A("transf ", 42);
-                    bErrorInTransform = true;
-                }
-                catch (const CannotRenameFile&) //ttt2 perhaps also NameNotFound, AlreadyExists, ...
-                {
-                //TRACER1A("transf ", 43);
-                    bErrorInTransform = true;
-                }
-                catch (const CannotCreateDir& ex)
-                {
-                //TRACER1A("transf ", 44);
-                    bErrorInTransform = true;
-                    m_strErrorDir = ex.m_strDir;
-                }
-                catch (const CannotCopyFile&)
-                {
-                //TRACER1A("transf ", 45);
-                    CB_ASSERT(false);
-                    //bErrorInTransform = true;
-                }
-//TRACER1A("transf ", 46);
+
                 if (bErrorInTransform)
                 {
                 //TRACER1A("transf ", 47);
@@ -545,6 +611,9 @@ bool Mp3Transformer::transform()
                             deleteFile(strProcName);
                             //TRACER1A("transf ", 50);
                         }
+                        catch (const exception& ex)
+                        { //ttt2
+                        }
                         catch (...)
                         { //ttt2 not sure what to do
                         }
@@ -565,7 +634,7 @@ bool Mp3Transformer::transform()
                     if ("*" != strNewOrigName && m_pCommonData->m_dirTreeEnum.isIncluded(strNewOrigName))
                     {
                     //TRACER1A("transf ", 54);
-                        m_vpAdd.push_back(new Mp3Handler(strNewOrigName, m_pCommonData->m_bUseAllNotes, m_pCommonData->getQualThresholds()));
+                        m_vpAdd.push_back(Mp3Handler::create(strNewOrigName, m_pCommonData->m_bUseAllNotes, m_pCommonData->getQualThresholds()));
                         //TRACER1A("transf ", 55);
                     }
                 }
@@ -583,14 +652,14 @@ bool Mp3Transformer::transform()
                     if (m_pCommonData->m_dirTreeEnum.isIncluded(strProcName))
                     {
                     //TRACER1A("transf ", 59);
-                        m_vpAdd.push_back(new Mp3Handler(strProcName, m_pCommonData->m_bUseAllNotes, m_pCommonData->getQualThresholds())); // !!! a new Mp3Handler is needed, because pNewHndl has an incorrect file name (but otherwise they should be identical)
+                        m_vpAdd.push_back(Mp3Handler::create(strProcName, m_pCommonData->m_bUseAllNotes, m_pCommonData->getQualThresholds())); // !!! a new Mp3Handler is needed, because pNewHndl has an incorrect file name (but otherwise they should be identical)
                         //TRACER1A("transf ", 60);
                     }
                     //TRACER1A("transf ", 61);
                 }
                 //TRACER1A("transf ", 62);
             }
-            catch (...)
+            catch (...) // ttt2 maybe catch std::exception
             {
             //TRACER1A("transf ", 63);
                 if (pNewHndl.get() == pOrigHndl)
@@ -603,6 +672,14 @@ bool Mp3Transformer::transform()
             }
         }
     }
+    catch (const exception& ex)
+    {
+        qDebug("Caught std::exception in Mp3TransformThread::transform()");
+        traceToFile("Caught std::exception in Mp3TransformThread::transform()", 0);
+        qDebug("%s", ex.what());
+        traceToFile(ex.what(), 0);
+        throw; // !!! needed to restore "erased" files when errors occur, because when an exception is thrown the destructors only get called if that exception is caught; so catching and rethrowing is not a "no-op"
+    }
     catch (...)
     {
     //TRACER1A("transf ", 66);
@@ -653,25 +730,32 @@ std::string Mp3Transformer::getError() const
 {
     if (!m_strErrorFile.empty())
     {
-        if (m_bWriteError)
+        if (!m_strOtherError.empty())
         {
-            return convStr(tr("There was an error writing to the following file:\n\n%1\n\nMake sure that you have write permissions and that there is enough space on the disk.\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))));
+            return convStr(tr("There was an error processing the following file:\n\n%1\n\n%2\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))).arg(convStr(m_strOtherError)));
         }
         else
         {
-            if (m_bFileChanged)
+            if (m_bWriteError)
             {
-                return convStr(tr("The file \"%1\" seems to have been modified since the last scan. You need to rescan it before continuing.\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))));
+                return convStr(tr("There was an error writing to the following file:\n\n%1\n\nMake sure that you have write permissions and that there is enough space on the disk.\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))));
             }
             else
             {
-                if (m_strErrorDir.empty())
+                if (m_bFileChanged)
                 {
-                    return convStr(tr("There was an error processing the following file:\n\n%1\n\nProbably the file was deleted or modified since the last scan, in which case you should reload / rescan your collection. Or it may be used by another program; if that's the case, you should stop the other program first.\n\nThis may also be caused by access restrictions or a full disk.\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))));
+                    return convStr(tr("The file \"%1\" seems to have been modified since the last scan. You need to rescan it before continuing.\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))));
                 }
                 else
                 {
-                    return convStr(tr("There was an error processing the following file:\n%1\n\nThe following folder couldn't be created:\n\n%2\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))).arg(convStr(toNativeSeparators(m_strErrorDir))));
+                    if (m_strErrorDir.empty())
+                    {
+                        return convStr(tr("There was an error processing the following file:\n\n%1\n\nProbably the file was deleted or modified since the last scan, in which case you should reload / rescan your collection. Or it may be used by another program; if that's the case, you should stop the other program first.\n\nThis may also be caused by access restrictions or a full disk.\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))));
+                    }
+                    else
+                    {
+                        return convStr(tr("There was an error processing the following file:\n%1\n\nThe following folder couldn't be created:\n\n%2\n\nProcessing aborted.").arg(convStr(toNativeSeparators(m_strErrorFile))).arg(convStr(toNativeSeparators(m_strErrorDir))));
+                    }
                 }
             }
         }
